/* $Workfile: OsaLinuxDmp.c $   $Revision: 3 $    $Modtime: 11/16/07 10:37a $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 2007, All Rights Reserved

*******************************************************************************/

/******************************************************************************
  
  Driver exported entry/exit functions.

  Some of the Linux kernel headers don't like to be compiled with g++ (C++)
  so we need a file that is pure C. This file has two purposes: it implements
  the two Linux module specific entry exit routines, init_module and
  cleanup_module, the file operation callbacks, and also some routines that's
  needed for proc operations.

******************************************************************************/
// This is the only place where this file should be included
// It's unlikely it will be needed anywhere else but if so
// __NO_VERSION__ must be defined before inclusion at the
// other places

// Only include in DaVinci
#ifdef DAVINCI


#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>	// struct tasklet_struct

#include <asm/uaccess.h>



#include "../OsaRtl.h"  // For OSA_DPRINTF

#include "OsaLinuxDmp.h"

#define PINMUX0     __REG(0x01c40000)
#define PINMUX1     __REG(0x01c40004)

#define OSA_DMP_LED0        0x80000000
#define OSA_DMP_LED1        0x00000001

DECLARE_WAIT_QUEUE_HEAD(dmp_isr_wq);

static spinlock_t g_irqLock;

static struct tasklet_struct g_sTasklets1; 
static unsigned long g_nWorkQueueData;
void (*g_pWorkqueueFunc)(IN unsigned long) = 0;


/******************************************************************************
  OsaLinuxDmpInit
    return 0->OK, Or Error
******************************************************************************/
int  
OsaLinuxDmpInit(void (*func)(unsigned long))
{
  g_pWorkqueueFunc = func;

  OSA_DPRINTF(("OsaLinuxDmpInit: WorkqueueFunc = 0x%X\n", g_pWorkqueueFunc));

  // Setup GPIO for LED write directly to the hardware register
	PINMUX0 |= OSA_DMP_LED0; // This is a physical address maybe it needs
	PINMUX1 |= OSA_DMP_LED1; // to be a virtual "io_p2v" call??

  return 0;
}


/******************************************************************************
  OsaLinuxDmpOpen
    return 0->OK, Or Error
******************************************************************************/
int 
OsaLinuxDmpOpen(void)
{
  int rc = 0;

  if(g_pWorkqueueFunc == 0) {
    OSA_DPRINTF(("OsaLinuxDmpOpen: Need to initialize DMP first\n"));
    rc = 1;
  }

  return rc;
}


/******************************************************************************
  OsaLinuxDmpClose
    return void
******************************************************************************/
void  
OsaLinuxDmpClose(void)
{
  g_pWorkqueueFunc = 0;

  return;
}


/******************************************************************************
  OsaLinuxDmpIrq
    return void
******************************************************************************/
static void 
OsaLinuxDmpIrq(unsigned long data)
{
	// Make sure only one is running
  spin_unlock(&g_irqLock);

  // Call DaVinci IRQ function
  if(g_pWorkqueueFunc != 0) {
    (*g_pWorkqueueFunc)(data);
  }

	spin_unlock(&g_irqLock);
}

/******************************************************************************
  OsaLinuxDmpWakeupIrq
    return void
******************************************************************************/
void 
OsaLinuxDmpWakeupIrq(void)
{

  g_nWorkQueueData++;

  // Setup a task to handle the IRQ
  tasklet_init((struct tasklet_struct *) &g_sTasklets1, 
               (void *) &OsaLinuxDmpIrq, 
               g_nWorkQueueData);
  tasklet_schedule((struct tasklet_struct *) &g_sTasklets1);
}


/******************************************************************************
  Set GPIO pin
    return 0=>OK, -1=>error
******************************************************************************/
int OsaLinuxDmpSetGpio(int gpioPin, int state)
{
  OSA_DPRINTF(("OsaLinuxDmpSetGpio: pin %d state %d\n", gpioPin, state));

  return gpio_set_direction(gpioPin, state); 
}


/******************************************************************************
  Get GPIO pin
    return 0=>OK, -1=>error
******************************************************************************/
int OsaLinuxDmpGetGpioValue(int gpioPin)
{
  OSA_DPRINTF(("OsaLinuxDmpGetGpioValue: pin %d state %d\n", gpioPin, state));

  return gpio_get_value(gpioPin);
}

/******************************************************************************
  Set GPIO pin
    return 0=>OK, -1=>error
******************************************************************************/
int OsaLinuxDmpSetGpioValue(int gpioPin, int value)
{
  OSA_DPRINTF(("OsaLinuxDmpGetGpioValue: pin %d state %d\n", gpioPin, state));

  return gpio_set_value(gpioPin, value);
}

/******************************************************************************
  OsaLinuxDmpReadReg32
    return 0=>OK, -1=>error
******************************************************************************/
int OsaLinuxDmpReadReg32(unsigned long address, unsigned long *value)
{
  int rc = 0;

  *value = *((volatile u32*) io_p2v(address));

  OSA_DPRINTF(("DmpReadReg32: Addr 0x%X value 0x%X\n", address, *value));

  return rc;
}

/******************************************************************************
  OsaLinuxDmpReadReg32
    return 0=>OK, -1=>error
******************************************************************************/
int OsaLinuxDmpWriteReg32(unsigned long address, unsigned long value)
{
  int rc = 0;

  OSA_DPRINTF(("DmpWriteReg32: Addr 0x%X value 0x%X\n", address, value));

  *((volatile u32*) io_p2v(address)) = value;

  return rc;
}

#endif // DAVINCI
